#include "StdAfx.h"
#include ".\clogger.h"

#include "LoggerSettings.h"

#include "CommonService.h"
#include "util.h"
#include "DynamicArray.h"
#include "cs_conv.h"
#include "mleak.h"

CLogger::CLogger(const CSessionList *pSessions)
: m_pSessions(pSessions)
{
}

CLogger::~CLogger(void)
{
}

//*******************************************************************
// public
//*******************************************************************

// Init() ɂ͊ɃZbVI[vĂ\̂
// Cleaup() 菈GłB
void CLogger::Init()
{
	CSessionList::session_iterator session;
	HNsmSession hSession;
	const CSessionInfo* si;
	CSessionInfo::member_iterator member;

	// SZbVɑ΂
	for( session = m_pSessions->begin(); session != m_pSessions->end(); session++ )
	{
		hSession = session->first;
		si = session->second;

		// I[v
		OnSessionOpen(hSession);

		if( IsLogOnce() )
		{
			MemberAdd(hSession, LLO_LOGACCOUNT);
			LogSessionOpen(hSession, LLO_LOGACCOUNT);
		}
		else
		{
			for( member = si->GetMembersBegin(); member != si->GetMembersEnd(); member++ )
			{
				MemberAdd(hSession, member->first.c_str());
			}
			for( member = si->GetMembersBegin(); member != si->GetMembersEnd(); member++ )
			{
				LogSessionOpen(hSession, member->first.c_str());
			}
		}
	}
}

void CLogger::Cleanup()
{
	CSessionList::session_iterator session;

	// SZbVɑ΂ăN[Y
	for( session = m_pSessions->begin(); session != m_pSessions->end(); session++ )
	{
		OnSessionClose(session->first);
	}
}

//*******************************************************************
// ̑ (protected)
//*******************************************************************

void CLogger::CreateDir(LPCTSTR lpLogFile) const
{
	int ret;

	ret = ::CreateDirAll(lpLogFile);
	if( ret != CDA_OK ) throw LEM_CANT_MAKE_DIR;
}

// G[o
//  lpErrMsg: G[
void CLogger::ErrorMsg(LPCTSTR lpErrMsg) const
{
	if( LOGGER_SETTING.IsNotifyError() ){
		TCHAR msg[INFOBUF_LEN];
		DWORD errCode;

		errCode = ::GetLastError();
		_snprintf_s(msg, sizeof(msg)/sizeof(TCHAR), _T("LoggerG[F\r\n%s(%d)"), lpErrMsg, errCode);

		NSM_SERVICE.DefaultNotify(msg);
	}
}

//*******************************************************************
// public Cxgnh (Kw1)
//*******************************************************************

// ZbVJn
void CLogger::OnSessionOpen(HNsmSession hSessionID)
{
	// ZbV쐬
	SessionOpen(hSessionID);

	if( IsLogOnce() )
	{
		MemberAdd(hSessionID, LLO_LOGACCOUNT);
		
		if( (*m_pSessions)[hSessionID]->GetStatus() == NMCS_CONNECTED )
		{
			LogSessionOpen(hSessionID, LLO_LOGACCOUNT);
		}
	}
}

// ZbVI
void CLogger::OnSessionClose(HNsmSession hSessionID)
{
	if( IsLogOnce() )
	{
		LogSessionClose(hSessionID, LLO_LOGACCOUNT);
		MemberRemove(hSessionID, LLO_LOGACCOUNT);
	}
	else
	{
		CSessionInfo::member_iterator i;
		const CSessionInfo *psi;

		psi = (*m_pSessions)[hSessionID];
		
		// ZbVN[YO
		for( i = psi->GetMembersBegin(); i != psi->GetMembersEnd(); i++ )
		{
			LogSessionClose(hSessionID, i->first.c_str());
			MemberRemove(hSessionID, i->first.c_str());
		}
	}

	// ZbV
	SessionClose(hSessionID);
}

// oǉ
void CLogger::OnMemberAdd(HNsmSession hSessionID, LPCSTR lpAccount)
{
	// OnXXX
	if( ! IsLogOnce() )
	{
		MemberAdd(hSessionID, lpAccount);
	}

	// LogXXX
	if( (*m_pSessions)[hSessionID]->GetStatus() == NMCS_CONNECTED )
	{
		if( ! IsLogOnce() )
		{
			LogSessionOpen(hSessionID, lpAccount);
		}
		LogMemberAdd(hSessionID, lpAccount);
	}
}

// o폜
void CLogger::OnMemberRemove(HNsmSession hSessionID, LPCSTR lpAccount)
{
	// LogXXX
	LogMemberRemove(hSessionID, lpAccount);

	if( ! IsLogOnce() )
	{
		LogSessionClose(hSessionID, lpAccount);
	}

	// OnXXX
	if( ! IsLogOnce() )
	{
		MemberRemove(hSessionID, lpAccount);
	}
}

// bZ[W
void CLogger::OnMessage(HNsmSession hSessionID, PMessageInfo pMsgInfo)
{
	LogMessage(hSessionID, pMsgInfo);
}

// ZbVύX
void CLogger::OnSessionConnected(HNsmSession SessionID)
{
	const CSessionInfo *psi;
	CSessionInfo::member_iterator i;

	psi = (*m_pSessions)[SessionID];
	if( IsLogOnce() )
	{
		LogSessionOpen(SessionID, LLO_LOGACCOUNT);
	}
	else
	{
		for( i = psi->GetMembersBegin(); i != psi->GetMembersEnd(); i++ )
		{
			LogSessionOpen(SessionID, i->first.c_str());
		}
	}
}

void CLogger::OnFileSessionStatusChange(HNsmFileSession hFSession, FileSessionStatus nStatus)
{
	LogFileSessionStatusChange(hFSession, nStatus);
}

//*******************************************************************
// 
//*******************************************************************

void CLogger::SessionOpen(HNsmSession /*hSessionID*/)
{
}

// ZbV̑SẴoZbV
void CLogger::SessionClose(HNsmSession /*hSessionID*/)
{
}

void CLogger::MemberAdd(HNsmSession /*hSessionID*/, LPCSTR /*lpAccount*/)
{
}

void CLogger::MemberRemove(HNsmSession /*hSessionID*/, LPCSTR /*lpAccount*/)
{
}


//*******************************************************************
// protected OCxgnh (Kw2)
//*******************************************************************

void CLogger::LogSessionOpen(HNsmSession hSession, LPCSTR lpAccount)
{
	_ASSERT( hSession != NULL );
	_ASSERT( lpAccount != NULL );
	DoDoLog(hSession, LMT_SESSION_OPEN, NULL, lpAccount, NULL);
}

void CLogger::LogSessionClose(HNsmSession hSession, LPCSTR lpAccount)
{
	_ASSERT( hSession != NULL );
	_ASSERT( lpAccount != NULL );
	DoDoLog(hSession, LMT_SESSION_CLOSE, NULL, lpAccount, NULL);
}

void CLogger::LogMemberAdd(HNsmSession hSession, LPCSTR lpAccount)
{
	_ASSERT( hSession != NULL );
	_ASSERT( lpAccount != NULL );
	DoDoLog(hSession, LMT_MEMBER_ADD, lpAccount, NULL, lpAccount);
}

void CLogger::LogMemberRemove(HNsmSession hSession, LPCSTR lpAccount)
{
	_ASSERT( hSession != NULL );
	_ASSERT( lpAccount != NULL );
	DoDoLog(hSession, LMT_MEMBER_REMOVE, lpAccount, NULL, lpAccount);
}

void CLogger::LogMessage(HNsmSession hSession, PMessageInfo pMsgInfo)
{
	_ASSERT( hSession != NULL );
	_ASSERT( pMsgInfo != NULL );
	DoDoLog(hSession, LMT_MESSAGE, NULL, NULL, NULL, pMsgInfo);
}

void CLogger::LogFileSessionStatusChange(HNsmFileSession hFSession, FileSessionStatus nStatus)
{
	_ASSERT( hFSession != NULL );
	TMessageInfo mi;
	LPCWSTR msg;
	HNsmSession hSession;

	switch( nStatus ){
	case NMFT_OPENNING:		msg = L"Jn"; break;
	case NMFT_INVITED:		msg = L"Jni肩̑Mj"; break;
	case NMFT_INVITING:		msg = L"Jni炩̑Mj"; break;
	case NMFT_ACCEPTED:		msg = L"]Jn"; break;
	case NMFT_CANCELED:		msg = L"]~"; break;
	case NMFT_TRANSFER_SUCCESS:	msg = L"]"; break;
	case NMFT_TRANSFER_FAILED:	msg = L"]s"; break;
	default:
		return;
	}

	memset(&mi, 0, sizeof(mi));
	mi.lpBody = msg;
	hSession = NSM_SERVICE.GetSessionFromFileSession(hFSession);
	_ASSERT( hSession != NULL );

	DoDoLog(hSession, LMT_FILESESSION_STATUSCHANGE, NULL, NULL, NULL, &mi, hFSession);
}

//*******************************************************************
// ʃOL^
//*******************************************************************

// virtual Kw3
// ʂ̏
void CLogger::DoDoLog(HNsmSession hSession, EMsgType nMsgType,
	LPCSTR lpFrom, LPCSTR lpTo, LPCSTR lpExcept,
	PMessageInfo pmi/*=NULL*/, HNsmFileSession hFSession/*=NULL*/)
{
	_ASSERT( hSession != NULL );
	LogMsgInfo lmi;
	CLogFormatter lf;

	// LogFormatter 
	if( pmi != NULL ){
		lf.Create((*m_pSessions)[hSession], pmi, hFSession);
	}else{
		lf.Create((*m_pSessions)[hSession], lpFrom, hFSession);
	}
	lf.EnableEscapeHtml(LOGGER_SETTING.IsEnableEscapeHtml());

	// LogMsgInfo 
	lmi.hSessionID = hSession;
	lmi.nMsgType = nMsgType;
	lmi.plf = &lf;

	// OL^
	DoLog(&lmi, lpTo, lpExcept);
}

// Kw4
// ݒɉċL^ΏۃAJEgƂ DoLogEach
void CLogger::DoLog(LogMsgInfo *plmi, LPCSTR lpTo, LPCSTR lpExcept)
{
	_ASSERT( plmi != NULL );

	if( IsLogOnce() )
		// OpX LogAccount ܂܂Ȃꍇ
		// 1񂾂L^
	{
		DoLogEach(LLO_LOGACCOUNT, plmi);
	}
	else if( lpTo != NULL )
		// 悪łȂꍇ
		// ̃OɋL^
	{
		DoLogEach(lpTo, plmi);
	}
	else
		// OׂAJEgłȂꍇ
		// OׂAJEgđSẴõOɋL^
	{
		const CSessionInfo* psi = (*m_pSessions)[plmi->hSessionID];
		CSessionInfo::member_iterator i;

		for( i = psi->GetMembersBegin(); i != psi->GetMembersEnd(); i++ )
		{
			if( (lpExcept == NULL) || (strcmp(lpExcept, i->first.c_str()) != 0) )
			{
				DoLogEach(i->first.c_str(), plmi);
			}
		}
	}
}

// virtual Kw5
// 1̃t@CɃOL^
//  lpAccount: ZbVɎQĂ郁o1l̃AJEg
//  plmi: L^ׂOɊւ
void CLogger::DoLogEach(LPCTSTR lpAccount, LogMsgInfo* plmi)
{
	_ASSERT( lpAccount != NULL );
	_ASSERT( plmi != NULL );
	CLogFile f;
	std::string logPath;
	std::wstring logText;

	plmi->plf->GetLogPath(logPath, LOGGER_SETTING.GetBaseDir(), lpAccount, LOGGER_SETTING.GetPathFormat());
	plmi->plf->GetLogText(logText, lpAccount, LOGGER_SETTING.GetLogFormat(plmi->nMsgType));

	try
	{
		CreateDir(logPath.c_str());

		f.Open(logPath.c_str());
		f.SeekEOF();
		f.WriteLog( LOGGER_SETTING.GetCharSet(), logText.c_str());
		f.Close();
	}
	catch(LPCTSTR lpErrMsg)
	{
		ErrorMsg(lpErrMsg);
	}
}

//*******************************************************************
// 
//*******************************************************************

