#pragma once
#include "clogger.h"
#include "consts.h"
#include "util.h"

class CFileMsgPool
{
private:
	typedef std::map<HNsmSession, MessagePool*> FileMsgPool;

public:
	typedef FileMsgPool::const_iterator pool_iterator;
	typedef FileMsgPool::mapped_type pool_type;

private:
	FileMsgPool m_pool;
	int m_nOffset;
	int m_nRegist;
	HANDLE m_hFile;

public:
	CFileMsgPool(LPCSTR lpFilePath)
	{
		m_nRegist = 0;
		m_nOffset = 0;

		CreateDir(lpFilePath);
		OpenFile(lpFilePath);
		m_nOffset = ::GetFileSize(m_hFile, NULL);
		if( m_nOffset < 0 ) throw LEM_CANT_GET_FSIZE;
	}
	~CFileMsgPool()
	{
		FileMsgPool::iterator i;
		for( i = m_pool.begin(); i != m_pool.end(); i++ ){
			delete i->second;
		}
		::CloseHandle(m_hFile);
	}
	static void CreateDir(LPCSTR lpFilePath)
	{
		int ret;

		ret = ::CreateDirAll(lpFilePath);
		if( ret != CDA_OK ) throw LEM_CANT_MAKE_DIR;
	}
	void OpenFile(LPCSTR lpFilePath)
	{
		m_hFile = ::CreateFile(lpFilePath, GENERIC_WRITE, FILE_SHARE_READ,
			NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if( m_hFile == INVALID_HANDLE_VALUE )
			throw LEM_CANT_OPEN_FILE;
	}
	void Regist(HNsmSession hSession)
	{
		_ASSERT( m_nRegist >= 0 );
		m_nRegist++;
		if( m_pool.find(hSession) == m_pool.end() )
			m_pool[hSession] = new MessagePool;
	}
	void UnRegist()
	{
		_ASSERT( m_nRegist > 0 );
		m_nRegist--;
	}
	bool IsRegisterd() const
	{
		_ASSERT( m_nRegist >= 0 );
		return m_nRegist > 0;
	}
	HANDLE GetFileHandle() const
	{
		return m_hFile;
	}
	pool_type& operator [](FileMsgPool::key_type key)
	{
		return m_pool[key];
	}
	pool_iterator begin() const
	{
		return m_pool.begin();
	}
	pool_iterator end() const
	{
		return m_pool.end();
	}
	int size() const
	{
		return m_pool.size();
	}
	int GetOffset() const
	{
		return m_nOffset;
	}
	bool Exists(FileMsgPool::key_type key) const
	{
		return m_pool.find(key) != m_pool.end();
	}
};
