#include "StdAfx.h"
#include "CommonService.h"
#include ".\loggermain.h"
#include "Logger.h"
#include "SessionLogger.h"
#include "MessageLogger.h"
#include "XMLSessionLogger.h"

#include "mleak.h"


#define LNMS_DEBUGLOG				NMM_ADDIN "/" PLUGIN_NAME "/DebugLog"


CLoggerMain::CLoggerMain(void)
{
	InitializeCriticalSection(&m_lock);
	m_pLogger = NULL;
//	new CMessageLogger(&m_sessions);
//	m_pLogger = new CSessionLogger(&m_sessions, false);
//	m_pLogger = new CNormalLogger(&m_sessions);
}

CLoggerMain::~CLoggerMain(void)
{
	delete m_pLogger;
	DeleteCriticalSection(&m_lock);
}

//*******************************************************************
// CxgtbN
//*******************************************************************

void CLoggerMain::OnInitialize()
{
	UpdateLogger();
}

// ZbVJn
void CLoggerMain::OnSessionOpen(HNsmSession hSessionID)
{
//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	Lock();
	RegistNewSession(hSessionID);
	m_pLogger->OnSessionOpen(hSessionID);
	Unlock();
}

// ZbVI
void CLoggerMain::OnSessionClose(HNsmSession hSessionID)
{
	Lock();
//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	m_pLogger->OnSessionClose(hSessionID);
	UnRegistSession(hSessionID);
	Unlock();
}

// oǉ
void CLoggerMain::OnMemberAdd(HNsmSession hSessionID, LPCSTR lpAccount)
{
	Lock();
//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	m_sessions[hSessionID]->AddMember(lpAccount);
	m_pLogger->OnMemberAdd(hSessionID, lpAccount);
	Unlock();
}

// o폜
void CLoggerMain::OnMemberRemove(HNsmSession hSessionID, LPCSTR lpAccount)
{
	Lock();
//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	m_pLogger->OnMemberRemove(hSessionID, lpAccount);
	m_sessions[hSessionID]->RemoveMember(lpAccount);
	Unlock();
}

// bZ[WM
void CLoggerMain::OnSendMessage(HNsmSession hSessionID, PMessageInfo pMsgInfo)
{
	Lock();
//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	m_pLogger->OnMessage(hSessionID, pMsgInfo);
	Unlock();
}

// bZ[WM
void CLoggerMain::OnReceiveMessage(HNsmSession hSessionID, PMessageInfo pMsgInfo)
{
	Lock();
//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	m_pLogger->OnMessage(hSessionID, pMsgInfo);
	Unlock();
}

// ZbVύX
void CLoggerMain::OnSessionInfoChange(HNsmSession SessionID, PNsmSessionInfo psi)
{
	bool bFirstConnected;

//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	if( psi->nInfoKey == NMSI_STATUS ){
		Lock();
		bFirstConnected = m_sessions[SessionID]->SetStatus(*(ConnectionStatus*)psi->lpInfo->lpBuffer);
		if( bFirstConnected )
		{
			m_pLogger->OnSessionConnected(SessionID);
		}
		Unlock();
	}
}

// oύX
void CLoggerMain::OnSessionMembersInfoChange(HNsmSession SessionID, PNsmMemberInfo pnmi)
{
//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	if( pnmi->nInfoKey == NMMI_NAME ){
		Lock();
		m_sessions[SessionID]->SetMemberName(pnmi->lpAccount, (LPCWSTR)pnmi->lpInfo->lpBuffer);
		Unlock();
	}
}

// t@CZbVύX
void CLoggerMain::OnFileSessionInfoChange(HNsmFileSession hFSession, PNsmFileSessionInfo pnfsi)
{
//	DEBUG_CODE(NSM_SERVICE.CallService(LNMS_DEBUGLOG, (WPARAM)__FUNCTION__));
	if( pnfsi->nInfoKey == NMFI_STATUS ){
		Lock();
		m_pLogger->OnFileSessionStatusChange(hFSession, *(FileSessionStatus*)pnfsi->lpInfo->lpBuffer);
		Unlock();
	}
}

void CLoggerMain::DoShowOptionDialog(HWND hWnd)
{
	int nLoggerType;
	std::string path, pathformat;

	nLoggerType = LOGGER_SETTING.GetLoggerType();
	path = LOGGER_SETTING.GetBaseDir();
	pathformat = LOGGER_SETTING.GetPathFormat();

	LOGGER_SETTING.ShowOptionDlg(hWnd);

	if( nLoggerType != LOGGER_SETTING.GetLoggerType() )
//		|| ((nLoggerType == LLT_SEP_MSG)
//			&& ((path != LOGGER_SETTING.GetBaseDir()) || (pathformat != LOGGER_SETTING.GetPathFormat()))) )
	{
		Lock();
		UpdateLogger();
		Unlock();
	}
}

void CLoggerMain::DoDebugLog(LPCTSTR str)
{
	SYSTEMTIME t;
	char path[MAX_PATH];
	FILE *f;

	::GetLocalTime(&t);
	strcpy_s(path, LOGGER_SETTING.GetBaseDir());
	strcat_s(path, "log.txt");

	fopen_s(&f, path, "at");
	_ASSERT(f != NULL);
	if( f == NULL ) return;

	fprintf(f, "%02d/%02d/%02d %02d:%02d:%02d.%03d| %s\n",
		t.wYear, t.wMonth, t.wDay, t.wHour, t.wMinute, t.wSecond,
		t.wMilliseconds, str);
	fclose(f);
}

void CLoggerMain::GetLogPath(HNsmConnection hConnection, LPCSTR targetAcc, std::string &path)
{
	CPathFormatter pf;

	pf.Create(hConnection);
	pf.GetLogPath(path, LOGGER_SETTING.GetBaseDir(), targetAcc, LOGGER_SETTING.GetPathFormat());
}

//*******************************************************************
// ̑
//*******************************************************************

void CLoggerMain::RegistNewSession(HNsmSession hSession)
{
	HNsmConnection hConnection;
	char buf[256];

	m_sessions.CreateSessionInfoEntry(hSession);

	hConnection = NSM_SERVICE.GetConnectionFromSession(hSession);
	m_sessions[hSession]->SetParentCon(hConnection);

	NSM_SERVICE.GetProtocol(hConnection, buf, sizeof(buf));
	m_sessions[hSession]->SetProtocol(buf);

	NSM_SERVICE.GetUserAccount(hConnection, buf, sizeof(buf));
	m_sessions[hSession]->SetMyAccount(buf);

	m_sessions[hSession]->SetStatus(NMCS_CONNECTING);
}

void CLoggerMain::UnRegistSession(HNsmSession hSession)
{
	m_sessions.DeleteSessionInfoEntry(hSession);
}

void CLoggerMain::UpdateLogger()
{
	if( m_pLogger != NULL ){
		m_pLogger->Cleanup();
		delete m_pLogger;
	}

	switch( LOGGER_SETTING.GetLoggerType() ){
	case LLT_MSN: m_pLogger = new CXMLSessionLogger(&m_sessions); break;
	case LLT_NOSEP: m_pLogger = new CLogger(&m_sessions); break;
	case LLT_SEP_SES: m_pLogger = new CSessionLogger(&m_sessions); break;
	case LLT_SEP_MSG: m_pLogger = new CMessageLogger(&m_sessions); break;
	default: _ASSERT(FALSE); break;
	}

	m_pLogger->Init();
}
