#include "StdAfx.h"
#include <shellapi.h>
#include <stdlib.h>
#include "CommonService.h"
#include "etc.h"
#include "LoggerPlugin.h"
#include "SmartLogFinder.h"
#include "LoggerSettings.h"
#include "general_strings.h"
#include "mleak.h"

#define LNMS_VIEWLASTLOG			NMM_ADDIN "/" PLUGIN_NAME "/UIService/Member/ViewLastLog"
#define LNMS_OPENLOGDIR				NMM_ADDIN "/" PLUGIN_NAME "/UIService/Main/OpenLogDir"
#define LNMS_DEBUGLOG				NMM_ADDIN "/" PLUGIN_NAME "/DebugLog"
#define LNMS_SHOWOPTIONDIALOG		"/ShowOptionDialog"
#define LNMS_GETLASTLOGINFO			"/GetLastLogInfo"
#define LNMS_GETLOGDIR				"/GetLogDir"
#define LNMS_VIEWLASTLOG_CAPTION	L"ŐV̉bOJ"
#define LNMS_OPENLOGDIR_CAPTION		L"bOtH_J"

const static char LPI_DESCRIPTION[] = "Session Message Logger";
const static char LPI_AUTHOR[]		= "fors";
const static char LPI_COPYRIGHT[]	= "http://cc.ii2.cc/~fors/sysworks/prog/regne/";
const static char LPI_PLUGINVER[]	= "0.4.0";

CLoggerPlugin CLoggerPlugin::m_singleton;
CLoggerSettings CLoggerSettings::m_singleton(PLUGIN_NAME);

//*******************************************************************
// RXgN^EfXgN^
//*******************************************************************
CLoggerPlugin::CLoggerPlugin(void)
: m_bServer(false)
{
	m_pApiVer		= NSM_API_VERSION;
//	m_pModuleName	= NMM_ADDIN "/" PLUGIN_NAME;
//	m_pTitle		= "Logger";
	m_pDescription	= LPI_DESCRIPTION;
	m_pAuthor		= LPI_AUTHOR;
	m_pCopyright	= LPI_COPYRIGHT;
	m_pPluginVer	= LPI_PLUGINVER;
}

CLoggerPlugin::~CLoggerPlugin(void)
{
}

void CLoggerPlugin::SetModuleInstance(HINSTANCE hInst)
{
	CNsmPlugin::SetModuleInstance(hInst);

	GetModuleName(&m_pluginName);
	m_moduleName = NMM_ADDIN "/";
	m_moduleName += m_pluginName;
	m_title = PLUGIN_NAME;
	if( m_pluginName != PLUGIN_NAME )
	{
		m_title += ": ";
		m_title += m_pluginName;
	}

	m_pModuleName	= m_moduleName.c_str();
	m_pTitle		= m_title.c_str();
}

//*******************************************************************
// K{֐
//*******************************************************************
int CLoggerPlugin::OnInitialize()
{
	if( NSM_SERVICE.GetService(LNMS_DEBUGLOG) == NULL )
	{
		m_bServer = true;
		NSM_SERVICE.CreateService(LNMS_DEBUGLOG, ServiceDebugLog);
		NSM_SERVICE.CreateService(LNMS_VIEWLASTLOG, ServiceViewLastLog);
		NSM_SERVICE.CreateService(LNMS_OPENLOGDIR, ServiceOpenLogDir);
	}
	CreateService(LNMS_SHOWOPTIONDIALOG, ServiceShowOptionDialog);
	CreateService(LNMS_GETLASTLOGINFO, ServiceGetLastLogInfo);
	CreateService(LNMS_GETLOGDIR, ServiceGetLogDir);

	LOGGER_SETTING.OnInitialize(m_pluginName.c_str());
	m_main.OnInitialize();

	return NMP_ALLRIGHT;
}

int CLoggerPlugin::OnSystemModulesLoaded()
{
	NSM_SERVICE.HookEvent(NME_SYSTEM_SESSION_MEMBERS_ADD, HookMemberAdd);
	NSM_SERVICE.HookEvent(NME_SYSTEM_SESSION_MEMBERS_REMOVE, HookMemberRemove);
	NSM_SERVICE.HookEvent(NME_SYSTEM_SESSION_SENDMESSAGE, HookSendMessage);
	NSM_SERVICE.HookEvent(NME_SYSTEM_SESSION_RECEIVEMESSAGE, HookReceiveMessage);
	NSM_SERVICE.HookEvent(NME_SYSTEM_SESSION_OPEN, HookSessionOpen);
	NSM_SERVICE.HookEvent(NME_SYSTEM_SESSION_CLOSE, HookSessionClose);
	NSM_SERVICE.HookEvent(NME_SYSTEM_SESSION_INFOCHANGE, HookSessionInfoChange);
	NSM_SERVICE.HookEvent(NME_SYSTEM_SESSION_MEMBERS_INFOCHANGE, HookSessionMembersInfoChange);
//	NSM_SERVICE.HookEvent(NME_SYSTEM_FILESESSION_OPEN, HookFileSessionOpen);
//	NSM_SERVICE.HookEvent(NME_SYSTEM_FILESESSION_CLOSE, HookFileSessionClose);
	NSM_SERVICE.HookEvent(NME_SYSTEM_FILESESSION_INFOCHANGE, HookFileSessionInfoChange);

	if( m_bServer )
	{
		CCommonService::module_list modules;
		CCommonService::module_list::iterator m;

		NSM_SERVICE.GetModuleList(modules);

		for( m = modules.begin(); m != modules.end(); ++m )
		{
			if( IsLoggerModule(*m) )
			{
				if( VersionMatch(*m) )
				{
					tstring::size_type pos = m->find("/");
					if( pos != tstring::npos )
					{
						m_children.push_back(m->c_str() + pos + 1);
					}
				}
				else
				{
					NSM_SERVICE.MsgBox(
						L"Logger x",
						L"قȂo[WLogger݂Ă܂B\n"
						L"Logger͐ɓ삵܂B"
					);
				}
			}
		}
		
	}

	return NMP_ALLRIGHT;
}

//*******************************************************************
// UIT[rX
//*******************************************************************

// uŐṼOJv
int CLoggerPlugin::DoViewLastLog(HNsmConnection hCnn, PNsmUIServiceInfo pUISInfo)
{

	switch( pUISInfo->nInfoKey ){
	case NMUI_GETCAPTION:
		wcsncpy_s((wchar_t*)pUISInfo->lpInfo->lpBuffer, pUISInfo->lpInfo->nBufferSize/sizeof(wchar_t), LNMS_VIEWLASTLOG_CAPTION, pUISInfo->lpInfo->nBufferSize/sizeof(wchar_t));
		pUISInfo->lpInfo->nType = NMIT_WIDESTRING;
		break;

	case NMUI_ONCLICK:
		{
			tstring path;
			GetLastLog(&path, hCnn, pUISInfo->lpAccount);
			ShellExec(path.c_str());
		}
		break;
	default:
		break;
	}
	return NMP_ALLRIGHT;
}

// uOtH_Jv
int CLoggerPlugin::DoOpenLogDir(HNsmConnection /*hConnectionID*/, PNsmUIServiceInfo pUISInfo)
{
	switch( pUISInfo->nInfoKey ){
	case NMUI_GETCAPTION:
		wcsncpy_s((wchar_t*)pUISInfo->lpInfo->lpBuffer, pUISInfo->lpInfo->nBufferSize/sizeof(wchar_t), LNMS_OPENLOGDIR_CAPTION, pUISInfo->lpInfo->nBufferSize/sizeof(wchar_t));
		pUISInfo->lpInfo->nType = NMIT_WIDESTRING;
		break;
	case NMUI_ONCLICK:
		{
			tstring path;
			GetLogDir(&path);
			ShellExec(path.c_str());
		}
		break;
	default:
		break;
	}
	return NMP_ALLRIGHT;
}

//*******************************************************************
// T[rX
//*******************************************************************
int CLoggerPlugin::DoGetLastLogInfo(LastLogInfo* pInfo)
{
	static LPCSTR LOG_TYPE[LFT_NUM] = {
		"U݊ ",
		"eLXg ",
		"ڍ ",
		"HTML ",
		"MSN ",
		"JX^ ",
	};
	std::string path, title, format;
	char proto[256], myacc[256];
	LPCSTR basedir;

	basedir = LOGGER_SETTING.GetBaseDir();
	NSM_SERVICE.GetProtocol(pInfo->hCnn, proto, sizeof(proto));
	NSM_SERVICE.GetUserAccount(pInfo->hCnn, myacc, sizeof(myacc));

	{
		SmartLogFinder finder(basedir, proto, myacc, pInfo->lpAcc);

		format = LOGGER_SETTING.GetPathFormat();
		finder.GetLatestLog(path, title, format);
	}

	if( (title.length() > 0) && (path.length() > 0) )
	{
		strncpy_s(pInfo->title, pInfo->title_len, LOG_TYPE[LOGGER_SETTING.GetFormatType()], pInfo->title_len);
		strncat_s(pInfo->title, pInfo->title_len, title.c_str(), pInfo->title_len);
		strncpy_s(pInfo->path, pInfo->path_len, path.c_str(), pInfo->path_len);

		return 1;
	}
	else
	{
		return 0;
	}
}

int CLoggerPlugin::DoGetLogDir(LPSTR buf, int size)
{
	strncpy_s(buf, size, LOGGER_SETTING.GetBaseDir(), size);
	return 0;
}

//*******************************************************************
// ̑
//*******************************************************************
// VFŊJ
//  lpFile: JpX
void CLoggerPlugin::ShellExec(LPCTSTR lpFile)
{
	TCHAR lpDir[MAX_PATH];
	LPSTR pos;

	lstrcpy(lpDir, lpFile);
	pos = _tcsrchr(lpDir, _T('\\'));
	if( pos != NULL ){
		*(pos + 1) = _T('\0');
		ShellExecute(NULL, "open", lpFile, NULL, lpDir, SW_SHOWNORMAL);
	}
}


void CLoggerPlugin::CreateService(LPCSTR name, TNsmServiceProc service)
{
	std::string serviceName;

	MakeServiceName(&serviceName, m_pluginName.c_str(), name);
	NSM_SERVICE.CreateService(serviceName.c_str(), service);
}

void CLoggerPlugin::GetModuleName(std::string* name)
{
	char buf[MAX_PATH];
	char file[_MAX_FNAME];

	::GetModuleFileName(GetModuleInstance(), buf, sizeof(buf));

	_splitpath_s(buf, NULL, 0, NULL, 0, file, _MAX_FNAME, NULL, 0);

	*name = file;
}

bool CLoggerPlugin::IsLoggerModule(const std::string& modName) const
{
	char buf[256];
	int ret;

	ret = NSM_SERVICE.GetPluginInfo(modName.c_str(), NMPI_AUTHOR, buf, sizeof(buf));
	if( (ret <= 0) || (strcmp(buf, LPI_AUTHOR) != 0) )
	{
		return false;
	}

	ret = NSM_SERVICE.GetPluginInfo(modName.c_str(), NMPI_DESCRIPTION, buf, sizeof(buf));
	if( (ret <= 0) || (strcmp(buf, LPI_DESCRIPTION) != 0) )
	{
		return false;
	}

	return true;
}

bool CLoggerPlugin::VersionMatch(const std::string& modName) const
{
	char buf[256];
	int ret;

	ret = NSM_SERVICE.GetPluginInfo(modName.c_str(), NMPI_PLUGINVER, buf, sizeof(buf));
	if( (ret <= 0) || (strcmp(buf, LPI_PLUGINVER) != 0) )
	{
		return false;
	}

	return true;
}

void CLoggerPlugin::MakeServiceName(std::string* pOut, LPCSTR module, LPCSTR name) const
{
	*pOut = NMM_ADDIN "/";
	*pOut += module;
	*pOut += name;
}


void CLoggerPlugin::GetLastLog(tstring* pPath, HNsmConnection hCnn, LPCSTR lpAcc) const
{
	ItemList list;

	// ŏIOW
	{
		StringList::const_iterator m;
		LastLogInfo lli;
		char title[MAX_PATH], path[MAX_PATH];
		tstring serviceName;
		int bSuccess;

		lli.hCnn = hCnn;
		lli.lpAcc = lpAcc;
		lli.title = title;
		lli.title_len = sizeof(title);
		lli.path = path;
		lli.path_len = sizeof(path);

		for( m = m_children.begin(); m != m_children.end(); ++m )
		{
			MakeServiceName(&serviceName, m->c_str(), LNMS_GETLASTLOGINFO);
			bSuccess = NSM_SERVICE.CallService(serviceName.c_str(),
				reinterpret_cast<WPARAM>(&lli) );
			if( bSuccess )
			{
				tstring title = *m;
				title += " (";
				title += lli.title;
				title += ")";
				list.push_back(std::make_pair(lli.path, title));
			}
		}
	}

	if( list.size() > 0 ){
		int idx = (list.size() > 1) ?
			GetUserSelect(list): 0;
		if( 0 <= idx && (unsigned int)idx < list.size() )
			pPath->swap(list[idx].first);
	}
}

void CLoggerPlugin::GetLogDir(tstring* pPath) const
{
	ItemList list;

	// OfBNgW
	{
		StringList::const_iterator m;
		char buf[MAX_PATH];
		tstring serviceName;


		for( m = m_children.begin(); m != m_children.end(); ++m )
		{
			MakeServiceName(&serviceName, m->c_str(), LNMS_GETLOGDIR);
			NSM_SERVICE.CallService(serviceName.c_str(),
				reinterpret_cast<WPARAM>(buf), sizeof(buf) );

			list.push_back(std::make_pair(buf, *m));
		}
	}

	if( list.size() > 0 ){
		int idx = (list.size() > 1) ?
			GetUserSelect(list): 0;
		if( 0 <= idx && (unsigned int)idx < list.size() )
			pPath->swap(list[idx].first);
	}
}

int CLoggerPlugin::GetUserSelect(const ItemList &list)
{
	MENUITEMINFO mii;
	int menuCount = 1;
	HMENU hMenu;
	BOOL ret;

	// ReLXgj[
	hMenu = ::CreatePopupMenu();

	memset(&mii, 0, sizeof(mii));
	mii.cbSize = sizeof(mii);
	mii.fMask = MIIM_TYPE|MIIM_ID;
	mii.fType = MFT_STRING;

	for( ItemList::const_iterator i = list.begin();
		i != list.end(); ++i )
	{
		if( i->first.length() > 0 && i->second.length() > 0 ){
			mii.dwTypeData = const_cast<LPSTR>(i->second.c_str());
			mii.cch = i->second.length();
			mii.wID = menuCount;
			ret = ::InsertMenuItem(hMenu, menuCount++, TRUE, &mii);
		}
	}

	// bZ[WMpEChE쐬
	HWND hWnd = CreateRcvWnd();

	// j[\
	POINT p;
	::GetCursorPos(&p);
	ret = ::TrackPopupMenu(hMenu,
		(TPM_RETURNCMD | TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON),
		p.x, p.y, 0, hWnd, 0);

	::DestroyMenu(hMenu);
	::DestroyWindow(hWnd);

	return (int)ret - 1;
}

HWND CLoggerPlugin::CreateRcvWnd()
{
	WNDCLASSEX wcex;

	memset(&wcex, 0, sizeof(wcex));
	wcex.cbSize = sizeof(wcex);
	wcex.lpfnWndProc = WndProc;
	wcex.hInstance = CNsmPlugin::GetModuleInstance();
	wcex.hbrBackground = (HBRUSH)COLOR_WINDOW;
	wcex.lpszClassName = "reciever";
	::RegisterClassEx(&wcex);
	return ::CreateWindow("reciever", NULL_STRING, NULL, 0, 0, 0, 0, NULL, NULL, CNsmPlugin::GetModuleInstance(), NULL);
}

LRESULT CALLBACK CLoggerPlugin::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return ::CallWindowProc(::DefWindowProc, hWnd, uMsg, wParam, lParam);
}
