#pragma once
#include "nsmPlugin.h"
#include "LoggerMain.h"
#include "StringList.h"

class CLoggerPlugin: public CNsmPlugin
{
// Singleton -->
private:
	static CLoggerPlugin m_singleton;

private:
	CLoggerPlugin(void);
public:
	~CLoggerPlugin(void);

public:
	static CLoggerPlugin& GetInstance()
	{
		return m_singleton;
	}
// <-- Singleton

protected:
	typedef std::vector<std::pair<std::string, std::string> > ItemList;

	struct LastLogInfo
	{
		HNsmConnection hCnn;
		LPCSTR lpAcc;

		LPSTR title;
		int title_len;
		LPSTR path;
		int path_len;
	};

private:
	CLoggerMain m_main;
	StringList m_children;
	std::string m_pluginName;
	std::string m_moduleName;
	std::string m_title;
	bool m_bServer;

public:
	void SetModuleInstance(HINSTANCE hInst);

public:
	DEFINE_EVENTHOOK2(MemberAdd, HNsmSession, LPSTR);
	DEFINE_EVENTHOOK2(MemberRemove, HNsmSession, LPSTR);
	DEFINE_EVENTHOOK2(SendMessage, HNsmSession, PMessageInfo);
	DEFINE_EVENTHOOK2(ReceiveMessage, HNsmSession, PMessageInfo);

	DEFINE_EVENTHOOK1(SessionOpen, HNsmSession);
	DEFINE_EVENTHOOK1(SessionClose, HNsmSession);
	DEFINE_EVENTHOOK2(SessionInfoChange, HNsmSession, PNsmSessionInfo);
	DEFINE_EVENTHOOK2(SessionMembersInfoChange, HNsmSession, PNsmMemberInfo);

//	DEFINE_EVENTHOOK1(FileSessionOpen, HNsmFileSession);
//	DEFINE_EVENTHOOK1(FileSessionClose, HNsmFileSession);
	DEFINE_EVENTHOOK2(FileSessionInfoChange, HNsmFileSession, PNsmFileSessionInfo);

	DEFINE_SERVICE2(ViewLastLog, HNsmConnection, PNsmUIServiceInfo);
	DEFINE_SERVICE2(OpenLogDir, HNsmConnection, PNsmUIServiceInfo);
	DEFINE_SERVICE1(DebugLog, LPCTSTR);
	DEFINE_SERVICE1(ShowOptionDialog, HWND);
	DEFINE_SERVICE1(GetLastLogInfo, LastLogInfo*);
	DEFINE_SERVICE2(GetLogDir, LPSTR, int);

private:
	int OnMemberAdd(HNsmSession hSessionID, LPSTR lpAccount)
		{ m_main.OnMemberAdd(hSessionID, lpAccount); return 0; }
	int OnMemberRemove(HNsmSession hSessionID, LPSTR lpAccount)
		{ m_main.OnMemberRemove(hSessionID, lpAccount); return 0; }
	int OnSendMessage(HNsmSession SessionID, PMessageInfo pMsgInfo)
		{ m_main.OnSendMessage(SessionID, pMsgInfo); return 0; }
	int OnReceiveMessage(HNsmSession SessionID, PMessageInfo pMsgInfo)
		{ m_main.OnReceiveMessage(SessionID, pMsgInfo); return 0; }
	int OnSessionOpen(HNsmSession SessionID)
		{ m_main.OnSessionOpen(SessionID); return 0; }
	int OnSessionClose(HNsmSession SessionID)
		{ m_main.OnSessionClose(SessionID); return 0; }
	int OnSessionInfoChange(HNsmSession hSession, PNsmSessionInfo pnsi)
		{ m_main.OnSessionInfoChange(hSession, pnsi); return 0; }
	int OnSessionMembersInfoChange(HNsmSession hSession, PNsmMemberInfo pnmi)
		{ m_main.OnSessionMembersInfoChange(hSession, pnmi); return 0; }
/*	int OnFileSessionOpen(HNsmFileSession hFSession)
		{ m_main.OnFileSessionOpen(hFSession); return 0; }
	int OnFileSessionClose(HNsmFileSession hFSession)
		{ m_main.OnFileSessionClose(hFSession); return 0; }
*/	int OnFileSessionInfoChange(HNsmFileSession hFSession, PNsmFileSessionInfo pfsi)
		{ m_main.OnFileSessionInfoChange(hFSession, pfsi); return 0; }

	void CreateService(LPCSTR name, TNsmServiceProc service);
	int DoDebugLog(LPCTSTR str)
		{ m_main.DoDebugLog(str); return 0; }
	int DoShowOptionDialog(HWND hWnd)
		{ m_main.DoShowOptionDialog(hWnd); return 0; }
	int DoViewLastLog(HNsmConnection hConnectionID, PNsmUIServiceInfo pUISInfo);
	int DoOpenLogDir(HNsmConnection hConnectionID, PNsmUIServiceInfo pUISInfo);
	int DoGetLastLogInfo(LastLogInfo* pInfo);
	int DoGetLogDir(LPSTR buf, int size);


	// J
	void ShellExec(LPCTSTR lpFile);
	bool IsLoggerModule(const std::string& modName) const;
	bool VersionMatch(const std::string& modName) const;
	void MakeServiceName(std::string* pOut, LPCSTR module, LPCSTR name) const;
	void GetLastLog(tstring* pPath, HNsmConnection hCnn, LPCSTR lpAcc) const;
	void GetLogDir(tstring* pPath) const;


private:
	static int GetUserSelect(const ItemList &list);
	static HWND CreateRcvWnd();
	static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static void GetModuleName(std::string* name);

// I[o[Ch
public:
	int OnInitialize();
	int OnSystemModulesLoaded();
};
