#include "stdafx.h"
#include <stdlib.h>
#include "LoggerSettings.h"
#include "optiondlg.h"
#include "resource.h"
#include "etc.h"
#include "NsmPlugin.h"
#include "general_strings.h"

// Logger Ini Entry
#define LIE_SETTINGS			_T("Settings")
#define LIE_NOTIFYERROR			_T("NotifyError")
#define LIE_CHARSET				_T("CharSet")
#define LIE_FORMATTYPE			_T("FormatType")
#define LIE_BASEDIR				_T("BaseDir")
#define LIE_PATHFORMAT			_T("PathFormat")
#define LIE_ENABLESYSTEMLOG		_T("EnableSystemLog")
#define LIE_ESCAPEHTML			_T("EscapeHtml")
#define LIE_LOGTYPE				_T("LogType")
#define LIE_SEPARATESESSION		_T("SeparateSession")
#define LIE_SAVEPERMESSAGE		_T("SavePerMessage")

// Logger Ini Entry
const LPCTSTR LIE_LOGFORMAT[] = {
	_T("MessageFormat"),
	_T("SessionOpenFormat"),
	_T("SessionCloseFormat"),
	_T("UserAddFormat"),
	_T("UserRemoveFormat"),
	_T("FileSessionFormat")
};

//*******************************************************************
// RXgN^EfXgN^
//*******************************************************************
CLoggerSettings::CLoggerSettings(LPCTSTR pluginName)
: COptionSettings(pluginName)
{
}

CLoggerSettings::~CLoggerSettings()
{
}

//*******************************************************************
// 
//*******************************************************************
void CLoggerSettings::OnInitialize(LPCSTR pluginName)
{
	SetPluginName(pluginName);
	ReadSettings();
}

//*******************************************************************
// ݒǂݍ
//*******************************************************************
// ݒt@C烁oϐɓǂݍ
void CLoggerSettings::ReadSettings()
{
	int value;

	// G[ʒm default = on
	value = m_ini.ReadInt(LIE_SETTINGS, LIE_NOTIFYERROR, 1);
	EnableNotifyError(value == 1);

	// Zbg default = Shift_JIS
	value = m_ini.ReadInt(LIE_SETTINGS, LIE_CHARSET, LCS_LOCALE);
	SetCharSet((value != LCS_UNICODE) ? LCS_LOCALE: LCS_UNICODE);

	// HTMLGXP[vs default = off
	value = m_ini.ReadInt(LIE_SETTINGS, LIE_ESCAPEHTML, 0);
	EnableEscapeHtml(value != 0);

	// ZbVǗ default = not separate
	value = m_ini.ReadInt(LIE_SETTINGS, LIE_SEPARATESESSION, false);
	EnableSeparateSession(value != false);
	value = m_ini.ReadInt(LIE_SETTINGS, LIE_SAVEPERMESSAGE, false);
	EnableSavePerMessage(value != false);

	// OL^` default = UbZ݊
	SetFormatType(m_ini.ReadInt(LIE_SETTINGS, LIE_FORMATTYPE, LFT_NISE));

	// pX default = ""
	{
		std::string fmt;
		m_ini.ReadString(fmt, LIE_SETTINGS, LIE_BASEDIR, NULL_STRING);
		SetBaseDir(fmt.c_str());
	}

	// pXtH[}bg default = LPF_DEFAULT
	{
		std::string fmt;
		m_ini.ReadString(fmt, LIE_SETTINGS, LIE_PATHFORMAT, LPF_DEFAULT);
		SetPathFormat(fmt.c_str());
	}

	// tH[}bg
	for( int i = 0; i < LMT_NUM; i++ )
	{
		std::string fmt;
		m_ini.ReadString(fmt, LIE_SETTINGS, LIE_LOGFORMAT[i], LLF_PRESET[LFT_NISE][i]);
		SetCustomFormat((EMsgType)i, fmt.c_str());
	}

	NormarizeBaseDir();
}

//*******************************************************************
// ݒ菑
//*******************************************************************
// oϐݒt@Cɏ
void CLoggerSettings::WriteSettings()
{
	// G[ʒm
	m_ini.Write(LIE_SETTINGS, LIE_NOTIFYERROR, IsNotifyError());

	// Zbg
	m_ini.Write(LIE_SETTINGS, LIE_CHARSET, GetCharSet());

	// HTMLGXP[vs
	m_ini.Write(LIE_SETTINGS, LIE_ESCAPEHTML, IsEnableEscapeHtml());

	// ZbVǗ
	m_ini.Write(LIE_SETTINGS, LIE_SEPARATESESSION, IsSeparateSession());
	m_ini.Write(LIE_SETTINGS, LIE_SAVEPERMESSAGE, IsSavePerMessage());

	// OL^`
	m_ini.Write(LIE_SETTINGS, LIE_FORMATTYPE, GetFormatType());	

	// pXo
	m_ini.Write(LIE_SETTINGS, LIE_BASEDIR, GetBaseDir());

	// pXtH[}bg
	m_ini.Write(LIE_SETTINGS, LIE_PATHFORMAT, GetPathFormat());

	// tH[}bgo
	for( int i = 0; i < LMT_NUM; i++ )
		m_ini.Write(LIE_SETTINGS, LIE_LOGFORMAT[i], GetCustomFormat((EMsgType)i));
}

//*******************************************************************
// ̑
//*******************************************************************
// ݒ_CAOI[v
//  hWnd: eEChẼnh
int CLoggerSettings::ShowOptionDlg(HWND hWnd)
{
	COptionDlg OpDlg;

	if( OpDlg.DoModal(CNsmPlugin::GetModuleInstance(), hWnd) == IDOK ){
		NormarizeBaseDir();
		WriteSettings();
	}
	return 0;
}

// OۑpX̍Ṓu\v
void CLoggerSettings::NormarizeBaseDir()
{
	if( m_lpBaseDir[m_lpBaseDir.length()-1] != _T('\\') )
		m_lpBaseDir += _T('\\');
}

