#pragma once
#include "optionSettings.h"
#include "consts.h"
#include "etc.h"

#define LOGGER_SETTING	(CLoggerSettings::GetInstance())

class CLoggerSettings: public COptionSettings
{
// singleton -->
private:
	static CLoggerSettings m_singleton;

private:
	CLoggerSettings(LPCTSTR pluginName);
	~CLoggerSettings(void);

public:
	static CLoggerSettings& GetInstance()
	{
		return m_singleton;
	}
// <-- singleton

public:
	// IvV_CAO
	int ShowOptionDlg(HWND hWnd);
	void OnInitialize(LPCSTR pluginName);

private:
	// oϐ
	bool m_bNotifyError;
	bool m_bEscapeHtml;
	bool m_bLoggingOnce;
	bool m_bSeparateSession;
	bool m_bSavePerMessage;

	int m_nFormatType;

	ECharSet m_nCharSet;
	tstring m_lpBaseDir;
	tstring m_lpPathFormat;

	tstring m_lpLogFormat[LMT_NUM];

	// ݒǂݍ
	void ReadSettings();
	// ݒt@C
	void WriteSettings();

	void NormarizeBaseDir();

//************
// oANZX
public:
	// G[ʒm
	bool IsNotifyError() const
	{
		return m_bNotifyError;
	}
	void CLoggerSettings::EnableNotifyError(bool bNotifyError)
	{
		m_bNotifyError = bNotifyError;
	}
	// Zbg
	void SetCharSet(ECharSet nCharSet)
	{
		m_nCharSet = nCharSet;
	}
	ECharSet GetCharSet() const
	{
		return m_nCharSet;
	}
	// HTMLGXP[vs
	bool IsEnableEscapeHtml() const
	{
		return m_bEscapeHtml || (m_nFormatType == LFT_MSN);
	}
	void EnableEscapeHtml(bool bEscapeHtml)
	{
		m_bEscapeHtml = bEscapeHtml;
	}
	// ZbVǗ
	bool IsSeparateSession() const
	{
		return m_bSeparateSession;
	}
	void EnableSeparateSession(bool bSeparateSession)
	{
		m_bSeparateSession = bSeparateSession;
	}
	bool IsSavePerMessage() const
	{
		return m_bSavePerMessage;
	}
	void EnableSavePerMessage(bool bSavePerMessage)
	{
		m_bSavePerMessage = bSavePerMessage;
	}
	// OL^`
	int GetFormatType() const
	{
		return m_nFormatType;
	}
	void SetFormatType(int nLogType)
	{
		m_nFormatType = nLogType;
	}


	int GetLoggerType() const
	{
		if( m_nFormatType == LFT_MSN )
			return LLT_MSN;

		if( m_bSeparateSession )
			return m_bSavePerMessage ? LLT_SEP_MSG: LLT_SEP_SES;

		return LLT_NOSEP;
	}
	bool IsLogggingOnce() const
	{
		return m_bLoggingOnce;
	}

	// x[XfBNg
#define LPF_TARGETACCOUNT_FULL		LPF_START_DELIMITER LPF_TARGETACCOUNT LPF_END_DELIMITER
	void SetBaseDir(LPCTSTR lpBaseDir)
	{
		m_lpBaseDir = lpBaseDir;
		if( m_lpBaseDir == _T("") ) m_lpBaseDir = GetMyDir();
	}
	LPCTSTR GetBaseDir() const
	{
		return m_lpBaseDir.c_str();
	}

	// pXtH[}bg
	void SetPathFormat(LPCTSTR lpFormat)
	{
		m_lpPathFormat = (strlen(lpFormat) == 0) ? LPF_DEFAULT: lpFormat;
		m_bLoggingOnce = (m_lpPathFormat.find(_T(LPF_TARGETACCOUNT_FULL)) == tstring::npos);
	}
	LPCTSTR GetPathFormat() const
	{
		return m_lpPathFormat.c_str();
	}

	// OtH[}bg
	LPCTSTR GetLogFormat(EMsgType nMsgType) const
	{
		if( nMsgType == LMT_SYSMESSAGE ) return "<!--Message-->\r\n";
		if( m_nFormatType == LFT_CUSTOM ) return m_lpLogFormat[nMsgType].c_str();
		return LLF_PRESET[m_nFormatType][nMsgType];
	}
	void SetCustomFormat(EMsgType nMsgType, LPCTSTR lpFormat)
	{
		m_lpLogFormat[nMsgType] = lpFormat;
	}
	LPCTSTR GetCustomFormat(EMsgType nMsgType) const
	{
		return m_lpLogFormat[nMsgType].c_str();
	}
};

