#include "StdAfx.h"
#include ".\messagelogger.h"
#include "LogFile.h"
#include "mleak.h"
#include "LoggerSettings.h"


//*******************************************************************
// 
//*******************************************************************

CMessageLogger::CMessageLogger(const CSessionList *pSessions)
: CLogger(pSessions)
{
}

CMessageLogger::~CMessageLogger(void)
{
	FileMsgPoolList::iterator pool;

	for( pool = m_msgPools.begin(); pool != m_msgPools.end(); pool++ )
		delete pool->second;
}

//*******************************************************************
// 
//*******************************************************************

void CMessageLogger::SessionOpen(HNsmSession SessionID)
{
	_ASSERT( m_filemap.find(SessionID) == m_filemap.end() );

	m_filemap[SessionID] = new AccToFileMap;
}

void CMessageLogger::SessionClose(HNsmSession SessionID)
{
	_ASSERT( m_filemap.find(SessionID) != m_filemap.end() );
	AccToFileMap* pafm;
	AccToFileMap::iterator map;

	pafm = m_filemap[SessionID];
	for( map = pafm->begin(); map != pafm->end(); map++ ){
		UnRegistSessionPool(SessionID, map->second.c_str());
	}
	delete pafm;
	m_filemap.erase(SessionID);
}

void CMessageLogger::MemberRemove(HNsmSession SessionID, LPCSTR lpAccount)
{
	_ASSERT( SessionRegisterd(SessionID, lpAccount) );
	std::string file;

	GetLogFileName(SessionID, lpAccount, file);
	UnRegistSessionPool(SessionID, file.c_str());
	m_filemap[SessionID]->erase(lpAccount);
}

//*******************************************************************
// 
//*******************************************************************

void CMessageLogger::DoLogEach(LPCTSTR lpAccount, LogMsgInfo* plmi)
{
	std::string file;
	std::wstring logText;
	AccToFileMap::iterator i;
	bool bSuccess;

	GetLogFileName(plmi->hSessionID, lpAccount, file);
	i = m_filemap[plmi->hSessionID]->find(lpAccount);
	if( i == m_filemap[plmi->hSessionID]->end() ){
		bSuccess = RegistSessionPool(plmi->hSessionID, file.c_str());
		if( ! bSuccess ) return;

		(*m_filemap[plmi->hSessionID])[lpAccount] = file;
	}else if( i->second != file ){
		UnRegistSessionPool(plmi->hSessionID, i->second.c_str());
		bSuccess = RegistSessionPool(plmi->hSessionID, file.c_str());
		if( ! bSuccess ) return;

		i->second = file;
	}

	plmi->plf->GetLogText(logText, lpAccount, LOGGER_SETTING.GetLogFormat(plmi->nMsgType));

	(*m_msgPools[file])[plmi->hSessionID]->logs.push_back(logText);
	FlushMsgPool(file, plmi->hSessionID);
}

void CMessageLogger::FlushMsgPool(std::string &file, HNsmSession /*SessionID*/)
{
	CLogFile f;
	CFileMsgPool *pfmp;
	CFileMsgPool::pool_iterator msgs;
	MsgLog *pml;
	MsgLog::const_iterator msg;

	pfmp = m_msgPools[file];

	// CX^XɊ֘At
	f.Attach(pfmp->GetFileHandle());

	// o
	try
	{
		f.Seek(pfmp->GetOffset(), FILE_BEGIN);
		for( msgs = pfmp->begin(); msgs != pfmp->end(); msgs++ ){
			pml = &(msgs->second->logs);
			for( msg = pml->begin(); msg != pml->end(); msg++ ){
				f.WriteLog(LOGGER_SETTING.GetCharSet(), msg->c_str());
			}
		}
	}
	catch(LPCTSTR lpErrMsg)
	{
		ErrorMsg(lpErrMsg);
	}

	// ֘At
	f.Detach();
}

//*******************************************************************
// 
//*******************************************************************

bool CMessageLogger::RegistSessionPool(HNsmSession SessionID, LPCSTR lpFile)
{
	FileMsgPoolList::const_iterator i;
	CFileMsgPool *pfmp;

	i = m_msgPools.find(lpFile);
	if( i == m_msgPools.end() ){
		try
		{
			pfmp = new CFileMsgPool(lpFile);
		}
		catch(LPCSTR lpErrMsg)
		{
			ErrorMsg(lpErrMsg);
			return false;
		}
		i = m_msgPools.insert(std::make_pair(lpFile, pfmp)).first;
	}
	i->second->Regist(SessionID);
	return true;
}

void CMessageLogger::UnRegistSessionPool(HNsmSession SessionID, LPCSTR lpFile)
{
	_ASSERT(SessionRegisterd(lpFile, SessionID));
	FileMsgPoolList::iterator i;

	i = m_msgPools.find(lpFile);
	if( i == m_msgPools.end() ) return;
	i->second->UnRegist();

	if( ! i->second->IsRegisterd() ){
		delete i->second;
		m_msgPools.erase(i);
	}
}
