#pragma once
#pragma warning( disable: 4702 )
	#include <map>
	#include <vector>
#pragma warning( default: 4702 )
#include <string>
#include "clogger.h"
#include "PathFormatter.h"
#include "LoggerSettings.h"
#include "FileMsgPool.h"

//*******************************************************************

/*
	CMessageLogger
		FileMsgPoolList = map(file => CFileMsgPool*)
			CFileMsgPool
				FileMsgPool = map(session => MessagePool*)
					MessagePool

	FileMsgPoolList{
		file => CFileMsgPool{
			Offset
			RegistCount
			FileMsgPool{
				Session => MessagePool
			}
		}
	}

*/

//*******************************************************************


//*******************************************************************

class CMessageLogger :
	public CLogger
{
	typedef std::map<std::string, CFileMsgPool*> FileMsgPoolList;

	typedef std::map<std::string , std::string> AccToFileMap;
	typedef std::map<HNsmSession, AccToFileMap*> SAtoFileMap;

protected:
	FileMsgPoolList m_msgPools;
	SAtoFileMap m_filemap;

public:
	CMessageLogger(const CSessionList *pSessions);
	virtual ~CMessageLogger(void);

protected:
	virtual void SessionOpen(HNsmSession SessionID);
	virtual void SessionClose(HNsmSession SessionID);
//	virtual void MemberAdd(HNsmSession hSessionID, LPCSTR lpAccount);
	virtual void MemberRemove(HNsmSession hSessionID, LPCSTR lpAccount);

	void DoLogEach(LPCSTR lpAccount, LogMsgInfo* plmi);
//	virtual bool IsLogOnce()

	bool RegistSessionPool(HNsmSession SessionID, LPCSTR lpFile);
	void UnRegistSessionPool(HNsmSession SessionID, LPCSTR lpFile);

	void FlushMsgPool(std::string &file, HNsmSession SessionID);

#ifdef _DEBUG
	bool SessionRegisterd(HNsmSession s, LPCSTR a) const
	{
		std::string file;
		GetLogFileName(s, a, file);
		return SessionRegisterd(file.c_str(), s);
	}
	bool SessionRegisterd(LPCSTR lpFile, HNsmSession s) const
	{
		return MemberRegisterd(lpFile) && m_msgPools.find(lpFile)->second->Exists(s);
	}
	bool MemberRegisterd(LPCSTR lpFile) const
	{
		return m_msgPools.find(lpFile) != m_msgPools.end();
	}
#endif

	void GetLogFileName(HNsmSession hSession, LPCSTR lpAccount, std::string &out) const
	{
		CPathFormatter pf;
		pf.Create((*m_pSessions)[hSession]);
		pf.GetLogPath(out, LOGGER_SETTING.GetBaseDir(), lpAccount, LOGGER_SETTING.GetPathFormat());
	}
};
