#pragma once
#include <string>
#pragma warning( disable: 4702 )
	#include <map>
#pragma warning( default: 4702 )
#include <utility>

class CSessionInfo
{
private:
	typedef std::map<std::string, std::wstring> AccountAndNameList;

public:
	typedef AccountAndNameList::const_iterator member_iterator;

private:
	std::string m_protocol;
	std::string m_myAccount;
	HNsmConnection m_hParentCon;
	ConnectionStatus m_nStatus;
	bool m_bOnceConnected;
	AccountAndNameList m_members;

public:
	CSessionInfo()
		: m_hParentCon(NULL)
		, m_nStatus(NMCS_DISCONNECTED)
		, m_bOnceConnected(false)
	{
	}
	void SetProtocol(LPCSTR proto)
	{
		_ASSERT( proto != NULL );
		m_protocol = proto;
	}
	void SetMyAccount(LPCSTR account)
	{
		_ASSERT( account != NULL );
		m_myAccount = account;
	}
	void SetParentCon(HNsmConnection hCon)
	{
		_ASSERT( hCon != NULL );
		m_hParentCon = hCon;
	}
	void AddMember(LPCSTR account)
	{
		_ASSERT( account != NULL );
		_ASSERT( ! MemberExists(account));
		m_members.insert(std::make_pair(account, L""));
	}
	void SetMemberName(LPCSTR account, LPCWSTR name)
	{
		_ASSERT( account != NULL );
		_ASSERT( name != NULL );
		_ASSERT(MemberExists(account));
		m_members[account] = name;
	}
	void RemoveMember(LPCSTR account)
	{
		_ASSERT( account != NULL );
		_ASSERT(MemberExists(account));
		m_members.erase(account);
	}
	bool SetStatus(ConnectionStatus nStatus)
	{
		m_nStatus = nStatus;
		if( (! m_bOnceConnected) && (nStatus == NMCS_CONNECTED) ){
			m_bOnceConnected = true;
			return true;
		}
		return false;
	}
	LPCSTR GetProtocol() const
	{
		return m_protocol.c_str();
	}
	LPCSTR GetMyAccount() const
	{
		return m_myAccount.c_str();
	}
	HNsmConnection GetParentCon() const
	{
		_ASSERT(m_hParentCon != NULL);
		return m_hParentCon;
	}
	LPCWSTR GetMemberName(LPCSTR account) const
	{
		_ASSERT( account != NULL );
		member_iterator i = m_members.find(account);
		return (i != m_members.end()) ? i->second.c_str(): NULL;
	}
	member_iterator GetMembersBegin() const
	{
		return m_members.begin();
	}

	member_iterator GetMembersEnd() const
	{
		return m_members.end();
	}
	ConnectionStatus GetStatus() const
	{
		return m_nStatus;
	}
	int GetNumOfMembers() const
	{
		return m_members.size();
	}

private:
	bool MemberExists(LPCSTR account) const
	{
		return m_members.find(account) != m_members.end();
	}
};
