#include "StdAfx.h"
#include <io.h>
#include <algorithm>
#include <functional>
#include ".\smartlogfinder.h"
#include "LoggerSettings.h"
#include "NsmPlugin.h"
#include "CommonService.h"
#include "general_strings.h"
#include "DynamicArray.h"

#define FINDER_BUF_SIZE	128

SmartLogFinder::SmartLogFinder(LPCSTR base, LPCSTR proto, LPCSTR myacc, LPCSTR tgacc)
: m_base(base)
, m_proto(proto)
, m_myacc(myacc)
, m_tgacc(tgacc ? tgacc: NULL_STRING)
{
}
/*
void SmartLogFinder::Find(std::string &path, HNsmConnection hCon, LPCSTR lpAcc)
{
	static LPCSTR LOG_TYPE[LFT_NUM] = {
		"U݊ (",
		"eLXg (",
		"ڍ (",
		"HTML (",
		"MSN (",
		"JX^ (",
	};
	ItemList list;
	BoolList flag = LOGGER_SETTING.GetLogFormatFlag();
	char proto[FINDER_BUF_SIZE], myacc[FINDER_BUF_SIZE];
	const EachSettings *es;
	LPCSTR basedir;
	std::string file, msg;

	// ʃf[^擾
	basedir = LOGGER_SETTING.GetCommonSettings()->baseDir.c_str();
	NSM_SERVICE.GetProtocol(hCon, proto, sizeof(proto));
	NSM_SERVICE.GetUserAccount(hCon, myacc, sizeof(myacc));

	// ⌟
	SmartLogFinder finder(basedir, proto, myacc, lpAcc);

	for( int i = 0; i < LFT_NUM; ++i ){
		if( flag.At(i) ){
			file = NULL_STRING;
			msg = NULL_STRING;
			es = LOGGER_SETTING.GetLoggerSettings((EFormatType)i);
			finder.GetLatestLog(file, msg, es->pathFormat);
			if( file.length() > 0 && msg.length() > 0 ){
				list.push_back(std::make_pair(file, LOG_TYPE[i] + msg + ")"));
			}
		}
	}

	if( list.size() > 0 ){
		int idx = (list.size() > 1) ?
			GetUserSelect(list): 0;
		if( 0 <= idx && (unsigned int)idx < list.size() )
			path.swap(list[idx].first);
	}
}
*/
void SmartLogFinder::GetLatestLog(std::string &path, 
	std::string &menu, const std::string &format)
{
	std::string replaced;

	if( ReplaceTag(replaced, format.c_str()) ){
		if( Dir(path, m_base, replaced.c_str()) ){
			MakeMenuString(&menu, (m_base + replaced).c_str(), path.c_str());
		}
	}
}

void SmartLogFinder::MakeMenuString(std::string* pMenu, LPCSTR format, LPCSTR str)
{
	int y, M, d, h, m, s;
	LPCSTR f_p, s_p;
	char buf[8];

	y = M = d = h = m = s = -1;

	f_p = format;
	s_p = str;
	for( ; *f_p && *s_p; ++f_p, ++s_p ){
		if( *f_p == '<' ){
			f_p++;

			int  num;
			int* pVal = NULL;

			switch( *f_p++ ){
			case 'y': pVal = &y; num = 4; break;
			case 'M': pVal = &M; num = 2; break;
			case 'd': pVal = &d; num = 2; break;
			case 'h': pVal = &h; num = 2; break;
			case 'm': pVal = &m; num = 2; break;
			case 's': pVal = &s; num = 2; break;
			default: _ASSERT(FALSE); return;
			}

			_ASSERT( pVal != NULL );
			*pVal = GetNumber(s_p, num);
			if( *pVal < 0 ){
				_ASSERT(FALSE);
				return;
			}

			s_p += num;
			if( *f_p++ != '>' ){
				_ASSERT(FALSE);
				return;
			}
		}
		if( *s_p != *f_p ) return;
	}

	if( y >= 0 ){
		sprintf_s(buf, "%dN", y);
		*pMenu += buf;
	}
	if( M >= 0 ){
		sprintf_s(buf, "%d", M);
		*pMenu += buf;
	}
	if( d >= 0 ){
		sprintf_s(buf, "%d", d);
		*pMenu += buf;
	}
	if( h >= 0 ){
		sprintf_s(buf, "%d", h);
		*pMenu += buf;
	}
	if( m >= 0 ){
		sprintf_s(buf, "%d", m);
		*pMenu += buf;
	}
	if( s >= 0 ){
		sprintf_s(buf, "%db", s);
		*pMenu += buf;
	}
}

bool SmartLogFinder::Dir(std::string &file, const std::string path, LPCSTR dir)
{
	bool ret;
	LPCSTR st, pos;

	st = strchr(dir, '\\');
	if( st == NULL ){
		// file
		pos = strchr(dir, '<');
		if( pos == NULL ){
			// no tag
			file = path + dir;
			return true;
		}else{
			// tag exists
			_finddata_t fd;
			intptr_t handle;
			std::string find = path + "*";
			StringList list;

			handle = _findfirst(find.c_str(), &fd);
			while( _findnext(handle, &fd) == 0 ){
				if( (fd.attrib & _A_SUBDIR) == 0 ){
					if( FormatMatch(dir, fd.name) ){
						list.push_back(fd.name);
					}
				}
			}
			_findclose(handle);

			if( list.size() > 0 ){
				std::sort(list.begin(), list.end(), StringCompare);
				file = path + list[0];

				return true;
			}else{
				return false;
			}
		}
	}else{
		// directory
		DynamicArray<char> buf(st - dir + 1);
		memcpy(buf, dir, st - dir);
		buf[st - dir] = '\0';
		
		st++;
		pos = strchr(buf, '<');
		if( pos == NULL ){
			// no tag
			return Dir(file, path + (char*)buf + '\\', st);
		}else{
			// tag exists
			_finddata_t fd;
			intptr_t handle;
			std::string find = path + "*";
			StringList list;

			handle = _findfirst(find.c_str(), &fd);
			while( _findnext(handle, &fd) == 0 ){
				if( fd.attrib & _A_SUBDIR ){
					if( FormatMatch(buf, fd.name) ){
						list.push_back(fd.name);
					}
				}
			}
			_findclose(handle);

			std::sort(list.begin(), list.end(), StringCompare);
			StringList::iterator i;
			ret = false;
			for( i = list.begin(); i != list.end(); ++i ){
				ret = Dir(file, path + *i + '\\', st);
				if( ret ) break;
			}
			return ret;
		}
	}
}

bool SmartLogFinder::ReplaceTag(std::string &out, LPCSTR in)
{
	LPTSTR fstart, fend, fprev;

	// tH[}bgRs[
	const int formatBufferSize = strlen(in)+1;
	DynamicArray<char> format(formatBufferSize);
	strcpy_s(format, formatBufferSize, in);

	// WJ
	fprev = format;
	while( (fstart = _tcsstr(fprev, LPF_START_DELIMITER)) != NULL
			&& (fend = _tcsstr(fstart+1, LPF_END_DELIMITER)) != NULL )
	{
		*fstart = _T('\0');
		*fend = _T('\0');
		out += fprev;

		fstart += _tcslen(LPF_START_DELIMITER);

		if( lstrcmp(fstart, LPF_DATE) == 0 )
		{	// t
			out += "<y>/<M>/<d>";
		}
		else if( lstrcmp(fstart, LPF_TIME) == 0 )
		{	// 
			out += "<h>:<m>:<s>";
		}
		else if( lstrcmp(fstart, LPF_YEAR) == 0 )
		{	// 
			out += "<y>";
		}
		else if( lstrcmp(fstart, LPF_MONTH) == 0 )
		{	// 
			out += "<M>";
		}
		else if( lstrcmp(fstart, LPF_DAY) == 0 )
		{	// 
			out += "<d>";
		}
		else if( lstrcmp(fstart, LPF_HOUR) == 0 )
		{	// 
			out += "<h>";
		}
		else if( lstrcmp(fstart, LPF_MINUTE) == 0 )
		{	// 
			out += "<m>";
		}
		else if( lstrcmp(fstart, LPF_SECOND) == 0 )
		{	// b
			out += "<s>";
		}
		else if( lstrcmp(fstart, LPF_PROTOCOL) == 0 )
		{	// vgR
			out += m_proto;
		}
		else if( lstrcmp(fstart, LPF_MYACCOUNT) == 0 )
		{	// ̃AJEg
			out += m_myacc;
		}
		else if( lstrcmp(fstart, LPF_TARGETACCOUNT) == 0 )
		{	// Ώۂ̃AJEg
			out += m_tgacc;
		}
		else
		{
			return false;
		}
		
		fprev = fend + _tcslen(LPF_END_DELIMITER);
	}
	out += fprev;
	return true;
}

bool SmartLogFinder::FormatMatch(LPCSTR format, LPCSTR str)
{
	LPCSTR f_p, s_p;

	f_p = format;
	s_p = str;
	for( ; *f_p && *s_p; ++f_p, ++s_p ){
		if( *f_p == '<' ){
			f_p++;

			int val, num;

			switch( *f_p++ ){
			case 'y': num = 4; break;
			case 'M': num = 2; break;
			case 'd': num = 2; break;
			case 'h': num = 2; break;
			case 'm': num = 2; break;
			case 's': num = 2; break;
			default:
				return false;
			}
			val = GetNumber(s_p, num);
			if( val < 0 ){
				return false;
			}

			s_p += num;
			if( *f_p++ != '>' ) return false;
		}
		if( *s_p != *f_p ) return false;
	}
	return *f_p == *s_p;
}

